Imports System.IO
''' <summary>
''' Archive plusieurs fichier dans un fichier
''' </summary>
''' <remarks></remarks>
Public Class T3DFiles

    'Structure du fichier:
    '-----------------------
    'Nombre magic (232) : 1 Octet 
    'Nombre fichier (Uint32) : 4 Octets 

    'Tableau d'info :taille variable

    'Position actuel: Uint32

    'Fichier coll bout  bout
    '-----------------------

    Private Magic As Byte = 232
    Private Count_Files As UInt32
    Private Structure Str_TabFiles
        Public Filename As String
        Public Length As UInt32
    End Structure

    Private Tab_Files() As Str_TabFiles
    Private Tab_FilesExt() As Str_TabFiles
    Private Tab_Length As Integer = -1
    Private Directory_Path As String

    Private Offset_File As UInt32
#Region "New"
    Public Sub New(ByVal Directory_File As String)
        Me.Directory_Path = Directory_File
    End Sub
#End Region

#Region "Add files"
    Public Sub Add_Files(ByVal Filename As String)
        'vrifie si le fichier existe
        If File.Exists(Directory_Path & "\" & Filename) = False Then Exit Sub
        Tab_Length += 1
        ReDim Preserve Tab_Files(Tab_Length)

        'stock le nom du fichier et a taille
        Tab_Files(Tab_Length).Filename = Filename
        Dim myFileInfo As New FileInfo(Directory_Path & "\" & Filename)
        Tab_Files(Tab_Length).Length = CUInt(myFileInfo.Length)
        myFileInfo = Nothing
    End Sub
#End Region

#Region "Save Archive"

    ''' <summary>
    ''' Sauvegarde le fichier archive
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <remarks></remarks>
    Public Sub Save(ByVal Archive_Filename As String)
        Dim OutFile As BinaryWriter = New BinaryWriter(File.Create(Archive_Filename))

        'Ecriture du nombre magic
        OutFile.Write(Magic)

        'Ecriture du nombre de fichier
        Count_Files = CUInt(Tab_Length + 1)
        OutFile.Write(Count_Files)

        'Ecriture du tableau
        For i As Integer = 0 To Tab_Length
            OutFile.Write(Tab_Files(i).Filename)
            OutFile.Write(Tab_Files(i).Length)
        Next

        'Ecriture de la position actuel pour determiner la position des fichiers
        OutFile.Write(CType(OutFile.Seek(0, SeekOrigin.Current), UInt32))

        'Ecriture des fichiers
        For i As Integer = 0 To Tab_Length
            Dim myfileAct As BinaryReader = New BinaryReader(File.Open(Directory_Path & "\" & Tab_Files(i).Filename, FileMode.Open))
            OutFile.Write(myfileAct.ReadBytes(CInt(Tab_Files(i).Length)))
            myfileAct.Close()
            myfileAct = Nothing
        Next

        OutFile.Close()
        OutFile = Nothing

    End Sub
#End Region

#Region "Extract Archive"
    ''' <summary>
    ''' Extrait l'archive dans un dossier
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <param name="Destination"></param>
    ''' <remarks></remarks>
    Public Sub Extract(ByVal Archive_Filename As String, ByVal Destination As String)
        If File.Exists(Archive_Filename) = False Then Exit Sub
        Dim Infile As BinaryReader = New BinaryReader(File.Open(Archive_Filename, FileMode.Open))

        If Infile.ReadByte <> Magic Then
            Infile.Close()
            MsgBox("Fichier invalide!")
            Infile = Nothing
            Exit Sub
        End If

        'rcupre le nombre de fichier
        Dim NbrFichier As UInt32 = Infile.ReadUInt32

        'Rcupre le nom et la taille des fichier
        ReDim Tab_FilesExt(CInt(NbrFichier - 1))
        For i As Integer = 0 To CInt(NbrFichier - 1)
            Tab_FilesExt(i).Filename = Infile.ReadString()
            Tab_FilesExt(i).Length = Infile.ReadUInt32
        Next

        'Recupere la position actuel        
        Offset_File = Infile.ReadUInt32

        For i As Integer = 0 To CInt(NbrFichier - 1)
            Dim buffertps(CInt(Tab_FilesExt(i).Length)) As Byte
            buffertps = Infile.ReadBytes(CInt(Tab_FilesExt(i).Length))

            'si le fichier est dans un rpertoire alors le crer s'il n'existe pas
            If Tab_FilesExt(i).Filename.IndexOf("\", 1) > 0 Then
                Dim dir_test As String = Tab_FilesExt(i).Filename.Substring(0, Tab_FilesExt(i).Filename.IndexOf("\", 1))
                If IO.Directory.Exists(Destination & "\" & dir_test) = False Then
                    IO.Directory.CreateDirectory(Destination & "\" & dir_test)
                End If
            End If

            'criture du fichier
            Dim sfile As BinaryWriter = New BinaryWriter(File.Create(Destination & "\" & Tab_FilesExt(i).Filename))
            sfile.Write(buffertps)
            sfile.Close()
            sfile = Nothing
        Next

        Infile.Close()
        Infile = Nothing

    End Sub

    ''' <summary>
    ''' Extrait seulement un fichier de l'archive
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <param name="Filename"></param>
    ''' <param name="Destination"></param>
    ''' <remarks></remarks>
    Public Sub Extract(ByVal Archive_Filename As String, ByVal Filename As String, ByVal Destination As String)
        If File.Exists(Archive_Filename) = False Then Exit Sub
        Dim Index_file As Integer

        Dim Infile As BinaryReader = New BinaryReader(File.Open(Archive_Filename, FileMode.Open))

        If Infile.ReadByte <> Magic Then
            Infile.Close()
            MsgBox("Fichier invalide!")
            Infile = Nothing
            Exit Sub
        End If
        Dim NbrFichier As UInt32 = Infile.ReadUInt32
        ReDim Tab_FilesExt(CInt(NbrFichier - 1))
        For i As Integer = 0 To CInt(NbrFichier - 1)
            Tab_FilesExt(i).Filename = Infile.ReadString()
            Tab_FilesExt(i).Length = Infile.ReadUInt32
            If Tab_FilesExt(i).Filename.ToLower = Filename Then Index_file = i
        Next

        'Recupere la position actuel        
        Offset_File = Infile.ReadUInt32

        'Decal l'index de depart pour le fichier voulu
        Dim IndexCumul As Integer = CInt(Offset_File + 4)
        If Index_file > 0 Then
            For i As Integer = 1 To Index_file
                IndexCumul += CInt(Tab_FilesExt(i - 1).Length)
            Next
        End If
        Infile.Close()
        Infile = Nothing

        'Rempli le buffer avec les donnes du fichier voulu
        Dim infile2 As BinaryReader = New BinaryReader(File.Open(Archive_Filename, FileMode.Open))
        infile2.BaseStream.Seek(IndexCumul, SeekOrigin.Begin)
        Dim buffertps(CInt(Tab_FilesExt(Index_file).Length - 1)) As Byte
        infile2.Read(buffertps, 0, buffertps.Length)

        'si le fichier est dans un rpertoire alors le crer s'il n'existe pas
        If Tab_FilesExt(Index_file).Filename.IndexOf("\", 1) > 0 Then
            Dim dir_test As String = Tab_FilesExt(Index_file).Filename.Substring(0, Tab_FilesExt(Index_file).Filename.IndexOf("\", 1))
            If IO.Directory.Exists(Destination & "\" & dir_test) = False Then
                IO.Directory.CreateDirectory(Destination & "\" & dir_test)
            End If
        End If

        'ecrit le fichier dans le dossier de destination
        Dim sfile As BinaryWriter = New BinaryWriter(File.Create(Destination & "\" & Tab_FilesExt(Index_file).Filename))
        sfile.Write(buffertps)
        sfile.Close()
        sfile = Nothing
        infile2.Close()
        infile2 = Nothing

    End Sub
#End Region

#Region "Get Files"

    ''' <summary>
    ''' rcupre le contenu d'un fichier texte de l'archive
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <param name="Filename"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Get_StringFiles(ByVal Archive_Filename As String, ByVal Filename As String) As String
        Dim streamR As StreamReader = New StreamReader(Get_StreamFiles(Archive_Filename, Filename))
        Return streamR.ReadToEnd
    End Function

    ''' <summary>
    ''' rcupre une image contenu dans l'archive
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <param name="Filename"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Get_ImageFiles(ByVal Archive_Filename As String, ByVal Filename As String) As Image
        Return Image.FromStream(Get_StreamFiles(Archive_Filename, Filename))
    End Function


    ''' <summary>
    ''' rcupre le contenu d'un fichier quelconque
    ''' </summary>
    ''' <param name="Archive_Filename"></param>
    ''' <param name="Filename"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Get_StreamFiles(ByVal Archive_Filename As String, ByVal Filename As String) As Stream
        Dim memstr As MemoryStream = New MemoryStream(Get_Files(Archive_Filename, Filename))
        Return memstr
    End Function

    Public Function Get_Files(ByVal Archive_Filename As String, ByVal Filename As String) As Byte()
        If File.Exists(Archive_Filename) = False Then Return Nothing
        Dim Index_file As Integer

        Dim Infile As BinaryReader = New BinaryReader(File.Open(Archive_Filename, FileMode.Open))

        If Infile.ReadByte <> Magic Then
            Infile.Close()
            MsgBox("Fichier invalide!")
            Infile = Nothing
            Return Nothing
        End If
        Dim NbrFichier As UInt32 = Infile.ReadUInt32
        ReDim Tab_FilesExt(CInt(NbrFichier - 1))
        For i As Integer = 0 To CInt(NbrFichier - 1)
            Tab_FilesExt(i).Filename = Infile.ReadString()
            Tab_FilesExt(i).Length = Infile.ReadUInt32

            'rcupre l'index du fichier voulu
            If Tab_FilesExt(i).Filename.ToLower = Filename Then Index_file = i
        Next

        'Rcupere la position actuel        
        Offset_File = Infile.ReadUInt32

        'Dcal l'index de depart pour le fichier voulu
        Dim IndexCumul As Integer = CInt(Offset_File + 4)
        If Index_file > 0 Then
            For i As Integer = 1 To Index_file
                IndexCumul += CInt(Tab_FilesExt(i - 1).Length)
            Next
        End If
        Infile.Close()
        Infile = Nothing

        'Rempli le buffer avec les donnes du fichier voulu
        Dim infile2 As BinaryReader = New BinaryReader(File.Open(Archive_Filename, FileMode.Open))
        infile2.BaseStream.Seek(IndexCumul, SeekOrigin.Begin)
        Dim buffertps(CInt(Tab_FilesExt(Index_file).Length - 1)) As Byte
        infile2.Read(buffertps, 0, buffertps.Length)
        infile2.Close()
        infile2 = Nothing
        Return buffertps

    End Function
#End Region

#Region "Dispose"
    Public Sub Dispose()
        Erase Tab_Files
        Tab_Files = Nothing
    End Sub
#End Region

End Class
